<?php
/**
 * ------------------------------------------------------------------------
 * JA T3v3 System Plugin
 * ------------------------------------------------------------------------
 * Copyright (C) 2004-2011 J.O.O.M Solutions Co., Ltd. All Rights Reserved.
 * @license - GNU/GPL, http://www.gnu.org/licenses/gpl.html
 * Author: J.O.O.M Solutions Co., Ltd
 * Websites: http://www.joomlart.com - http://www.joomlancers.com
 * ------------------------------------------------------------------------
 */

// No direct access
defined('_JEXEC') or die();

t3v3import ('extendable/extendable');

/**
 * T3V3Template class provides extended template tools used for T3v3 framework
 *
 * @package T3V3
 */
class T3v3Template extends ObjectExtendable
{
	/**
	 * Current template instance
	 *
	 * @var    string
	 */
	public $_tpl = null;

	/**
	 * Current template instance
	 *
	 * @var    string
	 */
	protected $_scripts = null;
	/**
	 * Class constructor
	 *
	 * @param   object  $template Current template instance
	 */
	public function __construct($template = null)
	{
		if ($template) {
			$this->_tpl = $template;			
			$this->_extend(array($template));
		}

		$this->_scripts = array();

		// merge layout setting
		$this->_tpl->params->loadString ($this->getParam('jat3v3_positions'));
	}

	/**
	 * get template parameter
	 *
	 * @param   $name  parameter name
	 * @param   $default  parameter default value
	 *
	 * @return   parameter value
	 */
	public function getParam ($name, $default = null) {
		return $this->_tpl->params->get ($name, $default);
	}
	
	public function setParam ($name, $value) {
		return $this->_tpl->params->set ($name, $value);
	}

	/**
	* Get current layout tpls
	*
	* @return string Layout name
	*/
	public function getLayout () {
		return JRequest::getCmd ('tmpl') ? JRequest::getCmd ('tmpl') : $this->getParam('mainlayout');
	}
 
	/**
	* Load block content
	*
	* @param $block string
	*     Block name - the real block is tpls/blocks/[blockname].php
	*
	* @return string Block content
	*/
	function loadBlock($block, $vars = array())
	{
		$path = T3V3Path::getPath ('tpls/blocks/'.$block.'.php');
		if ($path) {
			include $path;
		} else {
			echo "<div class=\"error\">Block [$block] not found!</div>";
		}
	}

	/**
	* Load block content
	*
	* @param $block string
	*     Block name - the real block is tpls/blocks/[blockname].php
	*
	* @return string Block content
	*/
	function loadLayout($layout)
	{
		$path = T3V3Path::getPath ('tpls/'.$layout.'.php', 'tpls/default.php');

		if (is_file ($path)) {
			include $path;
		} else {
			echo "<div class=\"error\">Layout [$layout] or [Default] not found!</div>";
		}
	} 

	/**
	* Load spotlight block
	*
	* @param $name string
	*     Name of the spotlight. Default will load positions base on this name: [name]-1, [name]-2...
	* @param $default_cols integer
	*     Default columns in the spotlight, changable in admin 
	*
	*/
	function spotlight($name, $default_pos, $default_width = '')
	{
		$vars['name'] = $name;
		$vars['pos'] = $this->getParam (T3V3_LAYOUT_PREFIX.$name.'_pos', $default_pos);
		$vars['width'] = $this->getParam (T3V3_LAYOUT_PREFIX.$name.'_width', $default_width);
		$this->loadBlock ('spotlight', $vars);
	} 


	/**
	* Get data property for layout - responsive layout
	*
	* @param $layout object
	*     Layout configuration
	* @param $col int
	*     Column number, start from 0
	*
	* @return string Block content
	*/
	function getData ($layout, $col) {
		$data = '';
		foreach ($layout as $device => $width) {
			if (!isset ($width[$col]) || !$width[$col]) continue;
			$data .= " data-$device=\"{$width[$col]}\"";
		}
		return $data;
	}

	/**
	* Get layout column class
	*
	* @param $layout object
	*     Layout configuration
	* @param $col int
	*     Column number, start from 0
	*
	* @return string Block content
	*/
	function getClass ($layout, $col) {
		$width = $layout->default;
		if (!isset ($width[$col]) || !$width[$col]) return "";
		return $width[$col];
	}

    /**
     * Wrap of document countModules function, get position from configuration before calculate
     */
    function countModules($positions)
    {
    	$pos = $this->getPosname ($positions);
        return $this->_tpl && method_exists($this->_tpl, 'countModules') ? $this->_tpl->countModules ($pos) : 0;
    }

	/**
	* Get position name
	*
	* @param $poskey string
	*     the key used in block
	*/
	function getPosname ($condition) {
		$operators = '(,|\+|\-|\*|\/|==|\!=|\<\>|\<|\>|\<=|\>=|and|or|xor)';
		$words = preg_split('# ' . $operators . ' #', $condition, null, PREG_SPLIT_DELIM_CAPTURE);
		for ($i = 0, $n = count($words); $i < $n; $i += 2)
		{
			// odd parts (modules)
			$name = strtolower($words[$i]);
			$words[$i] = $this->getParam (T3V3_LAYOUT_PREFIX.$name, $name);
		}

		$poss = implode(' ', $words);
		return $poss;
	}

	/**
	* echo position name
	*
	* @param $poskey string
	*     the key used in block
	*/
	function posname ($condition) {
		echo $this->getPosname ($condition);
	}

	/**
	* Alias of posname
	*
	*/
	function _p ($condition) {
		$this->posname ($condition);
	}

	/** 
	* echo position visibility (additinal class)
	*
	* @param $poskey string
	*     the key used in block
	*/
	function _c ($condition, $visibility = array()) {
		echo ' ' . implode(' ', $visibility);
	}

	/**
	* Add current template css base on template setting. 
	*
	* @param $name String
	*     file name, without .css
	*
	* @return string Block content
	*/
	function addCss ($name) {
		$devmode = $this->getParam('devmode', 0);
		$themermode = $this->getParam('themermode', 0);
		$theme = $this->getParam('theme', '');
		if (($devmode || ($themermode && defined ('T3V3_THEMER'))) && ($url = T3V3Path::getUrl('less/'.$name.'.less', '', true))) {
	    	t3v3import ('core/less');
	    	T3V3Less::addStylesheet ($url);
		} else {
			$url = T3V3Path::getUrl ('css/'.$name.'.css');
			// Add this css into template
			if ($url) {
				$this->addStyleSheet($url);
			}
		}
	}

  /**
   * Add T3v3 basic head 
   */
  function addHead () {
    $responsive = $this->getParam('responsive', 1);

    // BOOTSTRAP CSS
    $this->addCss ('bootstrap'); 
    // TEMPLATE CSS -->
    $this->addCss ('template'); 

    if ($responsive) {
      // BOOTSTRAP RESPONSIVE CSS
      $this->addCss ('bootstrap-responsive'); 
      // RESPONSIVE CSS
      $this->addCss ('template-responsive'); 
    }

    // Add scripts
    $this->addScript (T3V3_URL.'/bootstrap/js/jquery.js');
    $this->addScript (T3V3_URL.'/bootstrap/js/bootstrap.js');
    $this->addScript (T3V3_URL.'/js/touch.js');
    $this->addScript (T3V3_URL.'/js/script.js');
    
    if ($responsive) {
      $this->addScript (T3V3_URL.'/js/responsive.js');
    }
  }

  /**
   * Update head - detect if devmode or themermode is enabled and less file existed, use less file instead of css
   */
  function updateHead () {
  	$devmode = $this->getParam('devmode', 0);
  	$themermode = $this->getParam('themermode', 0) && defined ('T3V3_THEMER');
  	$theme = $this->getParam('theme', '');
  	// not in devmode and in default theme, do nothing
  	if (!$devmode && !$themermode && !$theme) return;

  	$doc = JFactory::getDocument();
  	$regex = '#'.T3V3_TEMPLATE_URL.'/css/([^/]*)\.css((\?|\#).*)?$#';
  	$stylesheets = array();
  	foreach ($doc->_styleSheets as $url => $css) {
  		// detect if this css in template css 
  		if (preg_match($regex, $url, $match)) {
  			$fname = $match[1];
  			if ($devmode || $themermode) {
	  			if (is_file (T3V3_TEMPLATE_PATH.'/less/'.$fname.'.less')) {
	  				// $doc->_styleSheets [JURI::current().'?t3action=lessc&s=templates/'.T3V3_TEMPLATE.'/less/'.$file] = $css;
	  				$newurl = JURI::current().'?t3action=lessc&s=templates/'.T3V3_TEMPLATE.'/less/'.$fname.'.less';
	  				$stylesheets[$newurl] = $css;
	  				continue;
	  			}
  			} 

			// check if css exists in current theme
			if (is_file (T3V3_TEMPLATE_PATH.'/css/themes/'.$theme.'/'.$fname.'.css')) {
				$newurl = T3V3_TEMPLATE_URL.'/css/themes/'.$theme.'/'.$fname.'.css';
				$stylesheets[$newurl] = $css;
				continue;
			}
  		}
  		$stylesheets[$url] = $css;
  	}
  	// update back
  	$doc->_styleSheets = $stylesheets;
  }

  function paramToStyle($style, $paramname = '', $isurl = false){
  	if($paramname == ''){
  		$paramname = $style;
  	}
  	$param = $this->getParam($paramname);
  	
  	if (!$param) return '';

  	if ($isurl) {
  		return "$style:url($param);";
  	} else {
  		return "$style:$param".(is_numeric($param) ? 'px;':';');
  	}
  }

}
?>